require 'spec_helper'

describe Admin::ProblemGroupsController do

  before do
    @user = Factory(:user)
    @teacher = Factory(:teacher)
    @admin = Factory(:admin)
  end

  admins = ADMINS

  describe "GET 'index'" do
    describe "by anonymous user" do
      before{ get 'index' }
      it{ response.should be_redirect }
    end
    describe "by non-admin user" do
      before do
        login_as(@user)
        get 'index'
      end
      it{ response.status.should == "403 Forbidden"}
      it{ response.should render_template('/admin/error') }
    end
    admins.each do |user|
      describe "by #{user}" do
        before do
          u = instance_variable_get("@#{user}")
          login_as(u)
          get 'index'
        end
        it{ response.should be_success }
        it{ response.should render_template('index') }
      end
    end
  end

  describe "GET 'show'" do
    describe "by anonymous user" do
      before{ get 'show' }
      it{ response.should be_redirect }
    end
    describe "by non-admin user" do
      before do
        login_as(@user)
        get 'show'
      end
      it{ response.status.should == "403 Forbidden"}
      it{ response.should render_template('/admin/error') }
    end
    admins.each do |user|
      describe "by #{user}" do
        before do
          problem_group = Factory(:problem_group)
          u = instance_variable_get("@#{user}")
          login_as(u)
          get 'show', :id => problem_group.id
        end
        it{ response.should be_success }
        it{ response.should render_template('show') }
      end
    end
  end

  describe "GET 'edit'" do
    describe "by anonymous user" do
      before{ get 'edit' }
      it{ response.should be_redirect }
    end
    describe "by non-admin user" do
      before do
        login_as(@user)
        get 'edit'
      end
      it{ response.status.should == "403 Forbidden"}
      it{ response.should render_template('/admin/error') }
    end
    admins.each do |user|
      describe "by #{user}" do
        before do
          problem_group = Factory(:problem_group)
          u = instance_variable_get("@#{user}")
          login_as(u)
          get 'edit', :id => problem_group.id
        end
        it{ response.should be_success }
        it{ response.should render_template('edit') }
      end
    end
  end

  describe "PUT 'update'" do
    describe "by anonymous user" do
      before{ put 'update' }
      it{ response.should be_redirect }
    end
    describe "by non-admin user" do
      before do
        login_as(@user)
        put 'update'
      end
      it{ response.status.should == "403 Forbidden"}
      it{ response.should render_template('/admin/error') }
    end
    admins.each do |user|
      describe "by #{user}" do
        before do
          problem_group = Factory(:problem_group)
          u = instance_variable_get("@#{user}")
          login_as(u)
          put('update', :id => problem_group.id,
              :problem_group => {
                :label => 'label',
                :description => 'description',
                :problems => problem_group.problems.map(&:id)
              })
        end
        it{ response.should be_redirect }
        it{ response.should redirect_to('/admin/problem_groups') }
      end
      describe "by #{user} failed" do
        before do
          problem_group = Factory(:problem_group)
          u = instance_variable_get("@#{user}")
          login_as(u)
          put('update', :id => problem_group.id,
              :problem_group => {
                :label => 'label',
                :description => 'description',
                :problems => []
              })
        end
        it{ response.should be_success }
        it{ response.should render_template('edit') }
      end
    end
  end

  describe "GET 'new'" do
    describe "by anonymous user" do
      before{ get 'new' }
      it{ response.should be_redirect }
    end
    describe "by non-admin user" do
      before do
        login_as(@user)
        get 'new'
      end
      it{ response.status.should == "403 Forbidden"}
      it{ response.should render_template('/admin/error') }
    end
    admins.each do |user|
      describe "by #{user}" do
        before do
          u = instance_variable_get("@#{user}")
          login_as(u)
          get 'new'
        end
        it{ response.should be_success }
        it{ response.should render_template('new') }
      end
    end
  end

  describe "POST 'create'" do
    describe "by anonymous user" do
      before{ post 'create' }
      it{ response.should be_redirect }
    end
    describe "by non-admin user" do
      before do
        login_as(@user)
        post 'create'
      end
      it{ response.status.should == "403 Forbidden"}
      it{ response.should render_template('/admin/error') }
    end
    admins.each do |user|
      describe "by #{user}" do
        before do
          problem_group = Factory(:problem_group)
          u = instance_variable_get("@#{user}")
          login_as(u)
          post('create',
              :problem_group => {
                :label => 'label',
                :description => 'description',
                :problems => problem_group.problems.map(&:id)
              })
        end
        it{ response.should be_redirect }
        it{ response.should redirect_to('/admin/problem_groups') }
      end
      describe "by #{user} failed" do
        before do
          u = instance_variable_get("@#{user}")
          login_as(u)
          put('create',
              :problem_group => {
                :label => 'label',
                :description => 'description',
                :problems => []
              })
        end
        it{ response.should be_success }
        it{ response.should render_template('new') }
      end
    end
  end

  describe "DELETE 'destroy'" do
    describe "by anonymous user" do
      before{ delete 'destroy' }
      it{ response.should be_redirect }
    end
    describe "by non-admin user" do
      before do
        login_as(@user)
        delete 'destroy'
      end
      it{ response.status.should == "403 Forbidden"}
      it{ response.should render_template('/admin/error') }
    end
    admins.each do |user|
      describe "by #{user}" do
        before do
          problem_group = Factory(:problem_group)
          u = instance_variable_get("@#{user}")
          login_as(u)
          delete "destroy", :id => problem_group.id
        end
        it{ response.should be_redirect }
        it{ response.should redirect_to('/admin/problem_groups') }
      end
    end
  end

  describe "XHR sort" do
    describe "by anonymous user" do
      before{ delete 'destroy' }
      it{ response.should be_redirect }
    end
    describe "by non-admin user" do
      before do
        login_as(@user)
        xhr :post, 'sort'
      end
      it{ response.status.should == "403 Forbidden"}
      it{ response.should render_template('/admin/error') }
    end
    admins.each do |user|
      describe "by #{user}" do
        before do
          u = instance_variable_get("@#{user}")
          login_as(u)
          xhr :post, 'sort', :position => []
        end
        it{ response.should be_success }
        it{ response.body.should match(/success/) }
      end
    end
  end
end
