require 'spec_helper'

describe Admin::ProblemsController do

  before do
    @user = Factory(:user)
    @admin = Factory(:admin)
    @teacher = Factory(:teacher)
  end

  UNITS = [:ordinary_arithmetic_level_1, :factorization_level_1]
  admins = ADMINS

  describe "GET 'index'" do
    describe "by anonymous user" do
      before{ get 'index' }
      it{ response.should be_redirect }
    end
    describe "by non-admin user" do
      before do
        login_as(@user)
        get "index"
      end
      it{ response.status.should == "403 Forbidden"}
      it{ response.should render_template('/admin/error') }
    end
    admins.each do |u|
      describe "by #{u}" do
        before do
          login_as(instance_variable_get("@#{u}"))
          get 'index'
        end
        it{ response.should be_success }
        it{ response.should render_template('index') }
      end
    end
  end

  describe "GET show" do
    describe "by anonymous" do
      before{ get "show", :id => 1 }
      it{ response.should be_redirect }
    end
    describe "by non-admin user" do
      before do
        login_as(@user)
        get "show", :id => 1
      end
      it{ response.status.should == "403 Forbidden"}
      it{ response.should render_template('/admin/error') }
    end
    admins.each do |user|
      UNITS.each do |unit|
        describe "by #{user}, #{unit}" do
          before do
            login_as(instance_variable_get("@#{user}"))
            Problem.should_receive(:find).with("1").and_return(Factory.build(unit))
            get "show", :id => 1
          end
          it{ response.should be_success }
          it{ response.should render_template('show') }
        end
      end
    end
  end

  describe "GET edit" do
    describe "by anonymous" do
      before{ get "edit", :id => 1 }
      it{ response.should be_redirect }
    end
    describe "by non-admin user" do
      before do
        login_as(@user)
        get "edit", :id => 1
      end
      it{ response.status.should == "403 Forbidden"}
      it{ response.should render_template('/admin/error') }
    end
    admins.each do |user|
      UNITS.each do |unit|
        describe "by #{user}" do
          before do
            login_as(instance_variable_get("@#{user}"))
            Problem.should_receive(:find).with("1").and_return(Factory.build(unit))
            get "edit", :id => 1
          end
          it{ response.should be_success }
          it{ response.should render_template('edit') }
        end
      end
    end
  end

  describe "GET 'new'" do
    describe "by anonymous user" do
      before{ get 'new' }
      it{ response.should be_redirect }
    end
    describe "by non-admin user" do
      before do
        login_as(@user)
        get "new"
      end
      it{ response.status.should == "403 Forbidden"}
      it{ response.should render_template('/admin/error') }
    end
    admins.each do |user|
      describe "by #{user}" do
        before do
          login_as(instance_variable_get("@#{user}"))
          get 'new'
        end
        it{ response.should be_success }
        it{ response.should render_template('new') }
      end
      UNITS.each do |unit|
        describe "by #{user}, #{unit}" do
          before do
            problem = Factory(unit)
            login_as(instance_variable_get("@#{user}"))
            xhr :post, 'new', :unit_id => problem.unit.id
          end
          it{ response.should be_success }
          it{ response.should render_template('/admin/problems/new_form') }
        end
      end
    end
  end

  describe "POST create" do
    describe "by anonymous user" do
      before{ post 'create' }
      it{ response.should be_redirect }
    end
    describe "by non-admin user" do
      before do
        login_as(@user)
        post 'create'
      end
      it{ response.status.should == "403 Forbidden"}
      it{ response.should render_template('/admin/error') }
    end
    admins.each do |user|
      UNITS.each do |unit|
        describe "by #{user}" do
          before do
            @problem = Factory(unit)
            @klass = @problem.unit.form_class
          end
          describe "with valid data" do
            before do
              login_as(instance_variable_get("@#{user}"))
              form = @klass.new
              form.should_receive(:valid?).and_return(true)
              @klass.should_receive(:new).and_return(form)
              Problem.should_receive(:create!).and_return(true)
              post 'create', :problem => { :unit_id => @problem.unit.id }
            end
            it{ response.should be_redirect }
            it{ response.should redirect_to('/admin/problems') }
          end
          describe "with invalid data" do
            before do
              login_as(instance_variable_get("@#{user}"))
              form = @klass.new
              form.should_receive(:valid?).and_return(false)
              @klass.should_receive(:new).and_return(form)
              post 'create', :problem => { :unit_id => @problem.unit.id }
            end
            it{ response.should be_success }
            it{ response.should render_template('/admin/problems/new_form') }
          end
        end
      end
    end
  end

  describe "PUT update" do
    describe "by anonymous user" do
      before{ post 'update' }
      it{ response.should be_redirect }
    end
    describe "by non-admin user" do
      before do
        login_as(@user)
        post 'update'
      end
      it{ response.status.should == "403 Forbidden"}
      it{ response.should render_template('/admin/error') }
    end
    admins.each do |u|
      describe "by #{u}" do
        describe "with valid data" do
          before do
            login_as(instance_variable_get("@#{u}"))
            problem = Factory.build(:ordinary_arithmetic_level_1)
            Problem.should_receive(:find).and_return(problem)
            form = OrdinaryArithmeticEditForm.new
            form.should_receive(:valid?).and_return(true)
            OrdinaryArithmeticEditForm.should_receive(:new).and_return(form)
            problem.should_receive(:update_attributes!).with(anything()).and_return(true)
            post 'update', :id => 1
          end
          it{ response.should be_redirect }
          it{ response.should redirect_to('/admin/problems') }
        end
        describe "with invalid data" do
          before do
            login_as(instance_variable_get("@#{u}"))
            problem = Factory.build(:ordinary_arithmetic_level_1)
            Problem.should_receive(:find).and_return(problem)
            form = OrdinaryArithmeticEditForm.new
            form.should_receive(:valid?).and_return(false)
            OrdinaryArithmeticEditForm.should_receive(:new).and_return(form)
            post 'update', :id => 1
          end
          it{ response.should be_success }
          it{ response.should render_template('/admin/problems/edit') }
        end
      end
    end
  end

  describe "DELETE destroy" do
    describe "by anonymous" do
      before{ delete "destroy", :id => 1 }
      it{ response.should be_redirect }
    end
    describe "by non-admin user" do
      before do
        login_as(@user)
        delete "destroy", :id => 1
      end
      it{ response.status.should == "403 Forbidden"}
      it{ response.should render_template('/admin/error') }
    end
    admins.each do |u|
      describe "by #{u}" do
        before do
          login_as(instance_variable_get("@#{u}"))
          problem = Factory.build(:ordinary_arithmetic_level_1)
          problem.should_receive(:destroy)
          Problem.should_receive(:find).with("1").and_return(problem)
          delete "destroy", :id => 1
        end
        it{ response.should be_redirect }
        it{ response.should redirect_to("/admin/problems") }
      end
    end
  end

end
