require 'spec_helper'

describe Report do
  before(:each) do
    @valid_attributes = {
      :user_id => 1,
      :problem_group_id => 1,
      :submitted => false
    }
    @klass = Report
    @basic = @klass.new(@valid_attributes)
  end

  describe "basic" do
    it{ @basic.should be_valid }
  end

  describe "validattion" do
    describe "user_id" do
      describe "is nil" do
        subject{ @basic.user_id = nil; @basic }
        it{ should_not be_valid }
      end
    end
    describe "problem_group_id" do
      describe "is nil" do
        subject{ @basic.problem_group_id = nil; @basic }
        it{ should_not be_valid }
      end
    end
  end

  describe "named_scope" do
    describe "by_user" do
      before do
        @user = Factory(:user)
        @admin = Factory(:admin)
        problem_group = Factory(:problem_group)
        Factory(:report, :user => @user, :problem_group => problem_group)
      end
      it{ @klass.by_user(@user).should have(1).report }
      it{ @klass.by_user(@admin).should have(0).report }
    end
    describe "by_date" do
      before do
        user = Factory(:user)
        admin = Factory(:admin)
        problem_group = Factory(:problem_group)
        Factory(:report, :user => user, :problem_group => problem_group, :created_at => 1.day.ago)
        Factory(:report, :user => user, :problem_group => problem_group)
        Factory(:report, :user => user, :problem_group => problem_group)
        Factory(:report, :user => user, :problem_group => problem_group, :created_at => 1.day.since)
      end
      it{ @klass.by_date(Date.today-2).should have(0).reports }
      it{ @klass.by_date(Date.today-1).should have(1).report }
      it{ @klass.by_date(Date.today).should have(2).reports }
      it{ @klass.by_date(Date.today+1).should have(1).report }
      it{ @klass.by_date(Date.today+2).should have(0).reports }
    end
    describe "by_range" do
      before do
        user = Factory(:user)
        admin = Factory(:admin)
        problem_group = Factory(:problem_group)
        Factory(:report, :user => user, :problem_group => problem_group, :created_at => 2.day.ago)
        Factory(:report, :user => user, :problem_group => problem_group, :created_at => 1.day.ago)
        Factory(:report, :user => user, :problem_group => problem_group)
        Factory(:report, :user => user, :problem_group => problem_group, :created_at => 1.day.since)
        Factory(:report, :user => user, :problem_group => problem_group, :created_at => 2.day.since)
      end
      it{ @klass.by_range(Date.today-1, Date.today).should have(2).reports }
    end
    describe "by_problem_group" do
      before do
        user = Factory(:user)
        admin = Factory(:admin)
        @problem_group = Factory(:problem_group)
        @problem_group2 = Factory(:problem_group, :label => "2",
                                  :problems => [Factory(:decimal_arithmetic_level_1)])
        Factory(:report, :user => user, :problem_group => @problem_group)
        Factory(:report, :user => user, :problem_group => @problem_group)
        Factory(:report, :user => user, :problem_group => @problem_group)
        Factory(:report, :user => user, :problem_group => @problem_group2)
        Factory(:report, :user => user, :problem_group => @problem_group2)
      end
      it{ @klass.by_problem_group(@problem_group.id).should have(3).reports }
    end
  end

  describe "summary" do
    describe "correct all" do
      before do
        user = Factory(:user)
        problem_group = Factory(:problem_group)
        problem = problem_group.problems.first
        attrs = Factory.build(:learning_result, "problem" => problem).attributes
        @report = Factory(:report, :user => user, :problem_group => problem_group)
        @report.learning_results.build(attrs)
        @report.learning_results.build(attrs)
        @report.learning_results.build(attrs)
        @report.save!
        @report.reload
      end
      it{ @report.summary.should == "3/3" }
    end
    describe "correct 2" do
      before do
        user = Factory(:user)
        problem_group = Factory(:problem_group)
        problem = problem_group.problems.first
        attrs = Factory.build(:learning_result, "problem" => problem).attributes
        @report = Factory(:report, :user => user, :problem_group => problem_group)
        @report.learning_results.build(attrs)
        @report.learning_results.build(attrs)
        @report.learning_results.build(attrs.update("answer" => "1"))
        @report.save!
        @report.reload
      end
      it{ @report.summary.should == "2/3" }
    end
    describe "wrong all" do
      before do
        user = Factory(:user)
        problem_group = Factory(:problem_group)
        problem = problem_group.problems.first
        attrs = Factory.build(:learning_result, "problem" => problem).attributes
        @report = Factory(:report, :user => user, :problem_group => problem_group)
        @report.learning_results.build(attrs.update("answer" => "1"))
        @report.learning_results.build(attrs.update("answer" => "1"))
        @report.learning_results.build(attrs.update("answer" => "1"))
        @report.save!
        @report.reload
      end
      it{ @report.summary.should == "0/3" }
    end
  end
end

